package saui_pr1;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

/**
 * This class defines the constants, methods and data memebers used for 
 * testing and calculating the coefficients of Fitts' Law This class 
 * implements the MouseListener methods except for mousePressed which 
 * its subclasses must implement.
 * 
 * @author Daniel Avrahami
 * @author Jason Hong
 */
public abstract class AbstractFittsLawTester
    implements MouseListener {

    //==========================================================================
    //===   CONSTANTS   ========================================================

    /**
     * This constant holds the width of the frame to be used. This const is
     * useful for finding the center of the screen.
     */
    public final int   MAX_WIDTH  = 800;

    /**
     * This constant holds the height of the frame to be used. This const is
     * useful for finding the center of the screen.
     */
    public final int   MAX_HEIGHT = 600;

    /**
     * This constant holds the maximum target radius to be used.
     */
    public final int   MAX_RADIUS = 50;

    /**
     * This constant holds the minimum target radius to be used.
     */
    public final int   MIN_RADIUS = 6;

    //===   CONSTANTS   ========================================================
    //==========================================================================


    

    
    //==========================================================================
    //===   INSTANCE VARIABLES   ===============================================
    
    /**
     * Array holding the time to hit the target as collected by the tester
     */
    protected double[] timeData;

    /**
     * Array holding the distances to the targets as collected by the tester
     */
    protected double[] distanceData;

    /**
     * Array holding the sizes of the targets as collected by the tester
     */
    protected double[] sizeData;

    /**
     * Number of trials to run
     */
    private int        trials;

    //===   INSTANCE VARIABLES   ===============================================
    //==========================================================================


    
    
    
    //==========================================================================
    //===   CONSTRUCTORS   =====================================================

    /**
     * Constructor that accepts the number of trials to run in the test and
     * initializes the data arrays to the number of trials
     * 
     * @param trials the number of trials to run the test
     */
    public AbstractFittsLawTester(int trials) {
        setTrials(trials);
        timeData     = new double[getTrials()];
        distanceData = new double[getTrials()];
        sizeData     = new double[getTrials()];
        createGUI();
    } // of constructor

    //===   CONSTRUCTORS   =====================================================
    //==========================================================================





    
    //==========================================================================
    //===   GUI METHODS   ======================================================

    /**
     * This method should create the interface used for collecting data. In
     * project 1 it should create a frame and one (or two buttons). One button
     * should be positioned in the center of the frame and should be the 
     * maximum target size allowed. (If you are using two buttons then the 
     * other button should be invisible) The last statement of this method 
     * should call the setVisible() method of the frame and set it to true.
     * 
     * <P>
     * STUDENTS SHOULD IMPLEMENT THIS METHOD.
     */
    public abstract void createGUI();

    //===   GUI METHODS   ======================================================
    //==========================================================================


    

    
    //==========================================================================
    //===   MOUSE METHODS   ====================================================

    /**
     * This method should capture the mouse press event, measure the time it
     * took to reach the button, generate a new size and position for the next
     * target and store all the data in timeData, distanceData, and sizeData
     * members.
     * 
     * <P>
     * STUDENTS SHOULD IMPLEMENT THIS METHOD.
     */
    public abstract void mousePressed(MouseEvent e);

    //---------------------------------------------------------------

    /* empty methods to satisfy MouseListener interface */
    public void mouseEntered(MouseEvent e) {
    } // of method

    public void mouseExited(MouseEvent e) {
    } // of method

    public void mouseClicked(MouseEvent e) {
    } // of method

    public void mouseReleased(MouseEvent e) {
    } // of method

    //===   MOUSE METHODS   ====================================================
    //==========================================================================

    
    
    
    
    //==========================================================================
    //===   ACCESSOR / MODIFIER METHODS   ======================================

    /**
     * Set the number of trials to run the test
     * 
     * @param trials the number of trials
     */
    public void setTrials(int trials) {
        if (trials > 0) {            
            this.trials = trials;
        }
    } // of method

    //---------------------------------------------------------------

    /**
     * Get the number of trials to run the test
     * 
     * @return int the number of trials
     */
    public int getTrials() {
        return trials;
    } // of method
    
    //===   ACCESSOR / MODIFIER METHODS   ======================================
    //==========================================================================


    
    
    
    //==========================================================================
    //===   FITTS LAW COEFFICIENT CALCULATION METHODS   ========================
    
    /**
     * This method performs the linear regression on the results and returns 
     * the two coefficients of Fitts's law
     * 
     * @param numOfTests the number of test cases to use (allows continuous 
     *        update of the coefficients)
     */
    public String calculateCoefficients(int numOfTests) {
        StringBuffer  strbuf       = new StringBuffer();  // string to return
        double[][]    combinedData = new double[numOfTests][2];

        strbuf.append("Time, Distance, Size, log2(D/S + 0.5)\n");
        
        // 1. Prepare the data structure for analysis in terms of Y and X
        //    for finding the coefficients in Y = b + aX
        for (int i = 0; i < numOfTests; i++) {
            combinedData[i][0] = timeData[i];
            combinedData[i][1] = Math.log(distanceData[i] / sizeData[i] + 0.5);

            // print out the values we collected and the log
            strbuf.append(timeData[i] + ", " + niceDouble(distanceData[i])
                    + ", " + sizeData[i] + ", "
                    + niceDouble(combinedData[i][1]) + "\n");
        }
        strbuf.append("\n");

        // 2. We'll compute the coeffs in two ways and output the results.
        //    If you're interested you can plot the data and the coeffs in excel
        double coef[] = linear_equation(combinedData, 1);
        strbuf.append("T = " + coef[0] + " + " + coef[1]
                + " * log2(D/S + 0.5)  [regression]\n");

        coef = quickRegress(combinedData);
        strbuf.append("T = " + coef[0] + " + " + coef[1]
                + " * log2(D/S + 0.5)  [quick regression]");

        // 3. Done.
        return strbuf.toString();
    } // of method

    //---------------------------------------------------------------

    /**
     * takes a double and rounds off to only 3 digits past the decimal point
     */
    private static double niceDouble(double original) {
        return (double) ((int) (original * 1000)) / 1000;
    } // of method

    //===   FITTS LAW COEFFICIENT CALCULATION METHODS   ========================
    //==========================================================================

    

    
    
    //==========================================================================
    //===   LINEAR REGRESSION METHODS   ========================================

    /*
     * The methods and members below this line are used in implementing the 
     * linear regression used to calculate the coefficients of the data 
     * collected. This code and comments were writting by Chris Rathman 
     * (ChrisRath@aol.com) and can be found here: 
     * http://www.angelfire.com/tx4/cus/regress/java.html
     * These methods can be used with any order equations.
     * 
     * If you would like to know a bit about linear regression you might want 
     * to look here: http://www.mste.uiuc.edu/patel/amar430/intro.html
     */

    double rsquared = 0;

    //---------------------------------------------------------------

    /**
     * Apply least squares to raw data to determine the coefficients for an
     * n-order equation: y = a0*X^0 + a1*X^1 + ... + an*X^n. Returns the
     * coefficients for the solved equation, given a number of y and x data
     * points. The rawData input is given in the form of {{y0, x0}, {y1,
     * x1},...,{yn, xn}}. The coefficients returned by the regression are {a0,
     * a1,...,an} which corresponds to {X^0, X^1,...,X^n}. The number of
     * coefficients returned is the requested equation order (norder) plus 1.
     */
    protected double[] linear_equation(double rawData[][], int norder) {
        double a[][]   = new double[norder + 1][norder + 1];
        double b[]     = new double[norder + 1];
        double term[]  = new double[norder + 1];
        double ysquare = 0;

        // step through each raw data entries
        for (int i = 0; i < rawData.length; i++) {

            // sum the y values
            b[0]    += rawData[i][0];
            ysquare += rawData[i][0] * rawData[i][0];

            // sum the x power values
            double xpower = 1;
            for (int j = 0; j < norder + 1; j++) {
                term[j]  = xpower;
                a[0][j] += xpower;
                xpower   = xpower * rawData[i][1];
            }

            // now set up the rest of rows in the matrix - multiplying each row
            // by each term
            for (int j = 1; j < norder + 1; j++) {
                b[j] += rawData[i][0] * term[j];
                for (int k = 0; k < b.length; k++) {
                    a[j][k] += term[j] * term[k];
                }
            }
        }

        // solve for the coefficients
        double coef[] = gauss(a, b);

        // calculate the r-squared statistic
        double ss = 0;
        double yaverage = b[0] / rawData.length;
        for (int i = 0; i < norder + 1; i++) {
            double xaverage = a[0][i] / rawData.length;
            ss += coef[i] * (b[i] - (rawData.length * xaverage * yaverage));
        }
        rsquared = ss / (ysquare - (rawData.length * yaverage * yaverage));

        // solve the simultaneous equations via gauss
        return coef;
    } // of method

    //---------------------------------------------------------------

    /**
     * It's just a standard gaussian technique for solving simultaneous
     * equations of the form: |A| = |B| * |C| where we know the values of |A|
     * and |B|, and we are solving for the coefficients in |C|
     */
    protected double[] gauss(double ax[][], double bx[]) {
        double a[][]  = new double[ax.length][ax[0].length];
        double b[]    = new double[bx.length];
        double pivot;
        double mult;
        double top;
        int    n      = b.length;
        double coef[] = new double[n];

        // copy over the array values - inplace solution changes values
        for (int i = 0; i < ax.length; i++) {
            for (int j = 0; j < ax[i].length; j++) {
                a[i][j] = ax[i][j];
            }
            b[i] = bx[i];
        }

        for (int j = 0; j < (n - 1); j++) {
            pivot = a[j][j];
            for (int i = j + 1; i < n; i++) {
                mult = a[i][j] / pivot;
                for (int k = j + 1; k < n; k++) {
                    a[i][k] = a[i][k] - mult * a[j][k];
                }
                b[i] = b[i] - mult * b[j];
            }
        }

        coef[n - 1] = b[n - 1] / a[n - 1][n - 1];
        for (int i = n - 2; i >= 0; i--) {
            top = b[i];
            for (int k = i + 1; k < n; k++) {
                top = top - a[i][k] * coef[k];
            }
            coef[i] = top / a[i][i];
        }
        return coef;
    } // of method

    //---------------------------------------------------------------

    /*
     * This is another method for calculating the coefficients. As you can see
     * it's much shorter but also not as accurate. This code was written by
     * Charles Stanton. I took it from the (very nice) applet found here:
     * http://www.math.csusb.edu/faculty/stanton/m262/regress/regress.html This
     * method can only be used with a equations of the form Y= b + aX.
     */

    /**
     * Perform a quick calculation of the coefficients using
     * the residual data for every point
     */
    protected double[] quickRegress(double[][] rawData) {
        double[] coef  = new double[2];
        int      sumxx = 0;
        int      sumyy = 0;
        int      sumxy = 0;
        int      sumx  = 0;
        int      sumy  = 0;
        double   Sxx, Sxy, Syy;

        for (int i = 0; i < rawData.length; i++) {
            sumx  += rawData[i][1];
            sumy  += rawData[i][0];
            sumxx += rawData[i][1] * rawData[i][1];
            sumyy += rawData[i][0] * rawData[i][0];
            sumxy += rawData[i][1] * rawData[i][0];
        }
        double n = (double) rawData.length;
        Sxx     = sumxx - sumx * sumx / n;
        Syy     = sumyy - sumy * sumy / n;
        Sxy     = sumxy - sumx * sumy / n;
        coef[1] = Sxy / Sxx;
        coef[0] = (sumy - coef[1] * sumx) / n;
        return coef;
    } // of method

    //===   LINEAR REGRESSION METHODS   ========================================
    //==========================================================================

} // of class