package saui_pr1;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import javax.swing.JButton;

/**
 * This class implements a round button.
 * It was taken from Sun's tutorial
 * http://developer.java.sun.com/developer/TechTips/1999/tt0826.html#tip1
 */
public class RoundButton 
    extends JButton {

    //==========================================================================
    //===   CONSTANTS   ========================================================

    /**
     * The color of the button when it is clicked on.
     */
    private static final Color COLOR_ARMED = Color.LIGHT_GRAY;
    
    //===   CONSTANTS   ========================================================
    //==========================================================================

    
    
    
    
    
    //==========================================================================
    //===   INSTANCE VARIABLES   ===============================================

    // Used for hit detection.
    Shape shape;

    //===   INSTANCE VARIABLES   ===============================================
    //==========================================================================





    //==========================================================================
    //===   CONSTRUCTORS   =====================================================
	
	/**
	 * A constructor for a RoundButton that takes a label as an argument
	 */
	public RoundButton(String label) {
		super(label);

		// 1. These statements enlarge the button so that it 
		//    becomes a circle rather than an oval.
		Dimension size = getPreferredSize();
		size.width  = Math.max(size.width, size.height);
        size.height = size.width;
		setPreferredSize(size);

		// 2. This call causes the JButton not to paint the
		//    background. This allows us to paint a round background.
		setContentAreaFilled(false);
    } // of constructor

    //---------------------------------------------------------------
	
	/**
	 * A constructor for a RoundButton with no label.
	 */
	public RoundButton() {
		this("");
    } // of constructor
	
    //---------------------------------------------------------------
	
	/**
	 * A constructor for a RoundButton with no label and with a specified 
     * color.
     * 
     * @param c is the color of the RoundButton.
	 */
	public RoundButton(Color c) {
		this();
		if (c != null) {
            setBackground(c);
        }
	} // of constructor
	
    //===   CONSTRUCTORS   =====================================================
    //==========================================================================
	

    

    
    //==========================================================================
    //===   GUI METHODS   ======================================================

	/**
	 * Paint the round background and label.
	 * This method overrides the paintComponent method of JButton
	 */
	protected void paintComponent(Graphics g) {
        // 1.1. Paint the button if it is being clicked on...
		if (getModel().isArmed()) {
			g.setColor(COLOR_ARMED);
		} 
        // 1.2. Otherwise make it the regular button color.
        else {
			g.setColor(getBackground());
		}

        // 2. Paint the component as an oval.
		g.fillOval(0, 0, getSize().width, getSize().height);

		// 3. This call will paint the label and the focus rectangle.
		super.paintComponent(g);
    } // of method

    //---------------------------------------------------------------

	/**
	 * Paint the border of the button using a simple stroke.
	 * This method overrides the paintBorder method of JButton
	 */
	protected void paintBorder(Graphics g) {
		g.setColor(getForeground());
		g.drawOval(0, 0, getSize().width-1, getSize().height-1);
    } // of method

    //---------------------------------------------------------------

	/**
	 * This method checks if the point x,y specified by the parameters
	 * is inside the roundbutton or not
	 * This method overrides the contains method of JButton.
     * 
	 * @return true if the point is inside the RoundButton, false otherwise
	 */
	public boolean contains(int x, int y) {
		// If the button has changed size, 
		// make a new shape object.
		if (shape == null || !shape.getBounds().equals(getBounds())) {
			shape = new Ellipse2D.Float(0, 0, getWidth(), getHeight());
		}

		return shape.contains(x, y);
	} // of method

    //===   GUI METHODS   ======================================================
    //==========================================================================

} // of class